<?php

######
###### classe responsable des plugins
######

class plugin{
	private $infos;
	private $config;
	private $name;
	private $hooks;
	private $isValid;
	private $isDefaultPlugin;
	private $titleTag;
	private $metaDescriptionTag;
	private $mainTitle;

	public function __construct($name, $config = array(), $infos = array(), $hooks = array()){
		$this->name = $name;
		$this->config = $config;
		$this->infos = $infos;
		$this->hooks = $hooks;
		$this->isValid = true;
		$this->isDefaultPlugin = ($name == DEFAULT_PLUGIN) ? true : false;
		$this->setTitleTag($infos['name']);
		$this->setMainTitle($infos['name']);
	}

	/*
	** getters
	*/
	public function getConfigVal($val){
		return $this->config[$val];
	}
	public function getInfoVal($val){
		return $this->infos[$val];
	}
	public function getName(){
		return $this->name;
	}
	public function getHooks(){
		return $this->hooks;
	}
	public function getIsDefaultPlugin(){
		return $this->isDefaultPlugin;
	}
	public function getTitleTag(){
		return $this->titleTag;
	}
	public function getMetaDescriptionTag(){
		return $this->metaDescriptionTag;
	}
	public function getMainTitle(){
		return $this->mainTitle;
	}

	/*
	** setters
	*/
	public function setConfigVal($k, $v){
		$this->config[$k] = $v;
		if($k == 'activate' && $v < 1 && $this->isDefaultPlugin) $this->isValid = false;
	}
	public function setTitleTag($val){
		if($this->isDefaultPlugin) $val = getConfVal('core', 'siteName').' | '.trim($val);
		else $val = $val.' | '.getConfVal('core', 'siteName');
		if(mb_strlen($val) > 50) $val = mb_strcut($val, 0, 50).'...';
		$this->titleTag = $val;
	}
	public function setMetaDescriptionTag($val){
		$val = trim($val);
		if(mb_strlen($val) > 150) $val = mb_strcut($val, 0, 150).'...';
		$this->metaDescriptionTag = $val;
	}
	public function setMainTitle($val){
		$val = trim($val);
		$this->mainTitle = $val;
	}

	/*
	** retourne le chemin du fichier lib
	*/
	public function getLibFile(){
		if(file_exists(ROOT.'plugin/'.$this->name.'/lib.php')) return ROOT.'plugin/'.$this->name.'/lib.php';
		return false;
	}
	/*
	** retourne le chemin du fichier front
	*/
	public function getFrontFile(){
		if(file_exists(ROOT.'plugin/'.$this->name.'/front.php')) return ROOT.'plugin/'.$this->name.'/front.php';
		return false;
	}
	/*
	** retourne le chemin du fichier admin
	*/
	public function getAdminFile(){
		if(file_exists(ROOT.'plugin/'.$this->name.'/admin.php')) return ROOT.'plugin/'.$this->name.'/admin.php';
		return false;
	}
	/*
	** retourne le chemin du fichier CSS
	*/
	public function getCssFile(){
		if(file_exists(ROOT.'plugin/'.$this->name.'/'.$this->name.'.css')) return ROOT.'plugin/'.$this->name.'/'.$this->name.'.css';
		return false;
	}
	/*
	** retourne le chemin du fichier JS
	*/
	public function getJsFile(){
		if(file_exists(ROOT.'plugin/'.$this->name.'/'.$this->name.'.js')) return ROOT.'plugin/'.$this->name.'/'.$this->name.'.js';
		return false;
	}
	/*
	** retourne le chemin du repertoire de donnees
	*/
	public function getDataPath(){
		$path = ROOT.'data/plugin/'.$this->name.'/';
		if(is_dir($path)) return $path;
		return false;
	}
	/*
	** determine si le plugin est installé / a jour
	*/
	public function isInstalled(){
		// on check si la config existe
		if(count($this->config) < 1) return false;
		// on compare la config initiale avec la config actuelle
		$currentConfig = implode(',', array_keys($this->config));
		$initialConfig = implode(',', array_keys(call_user_func($this->name.'Config')));
		if($currentConfig != $initialConfig) return false;
		return true;
	}

	/*
	** liste les plugins
	*/
	public static function listAll(){
		$data = array();
		$dataNotSorted = array();
		foreach(scandir(ROOT.'plugin/') as $dir){
			if(is_dir(ROOT.'plugin/'.$dir) && $dir[0] != '.') $dataNotSorted[$dir] = json_decode(@file_get_contents(ROOT.'data/plugin/'.$dir.'/config.txt'), true);
		}
		$dataSorted = utilSort2DimArray($dataNotSorted, 'priority', 'num');
		foreach($dataSorted as $plugin=>$config){
			$data[] = plugin::create($plugin);
		}
		return $data;
	}
	/*
	** liste les plugins actifs
	*/
	public static function listActive(){
		$data = plugin::listAll();
		foreach($data as $k=>$plugin){
			if($plugin->getConfigVal('activate') < 1) unset($data[$k]);
		}
		return $data;
	}
	/*
	** cree un objet plugin
	*/
	public static function create($name){
		$config = array();
		if(file_exists(ROOT.'data/plugin/'.$name.'/config.txt')) $config = json_decode(@file_get_contents(ROOT.'data/plugin/'.$name.'/config.txt'), true);
		$infos = @call_user_func($name.'Infos');
		$hooks = @call_user_func($name.'Hooks');
		return new plugin($name, $config, $infos, $hooks);
	}
	/*
	** installe / update un plugin
	*/
	public static function install($name){
		@mkdir(ROOT.'data/plugin/'.$name.'/', 0777);
		@chmod(ROOT.'data/plugin/'.$name.'/', 0777);
		@file_put_contents(ROOT.'data/plugin/'.$name.'/config.txt', json_encode(call_user_func($name.'Config')), 0666);
		@chmod(ROOT.'data/plugin/'.$name.'/config.txt', 0666);
		if(function_exists($name.'Install')) call_user_func($name.'Install');
		if(!file_exists(ROOT.'data/plugin/'.$name.'/config.txt')) return false;
		return true;
	}
	/*
	** enregistre la config d'un plugin
	*/
	public static function saveConfig($obj){
		if($obj->isValid && $path = $obj->getDataPath()){
			if(@file_put_contents($path.'config.txt', json_encode($obj->config), 0666)) return true;
		}
		return false;
	}
}
?>
